/**
 * Super simple and lightweight function chain.
 * 
 * Usage:
 * 	
 * 	// Start new chain.
 * 	var chain = new Chainer();
 * 	
 * 	// Add callbacks to chain.
 * 	chain.add(function(){
 * 		console.log('First!');
 * 		chain.next();
 * 	});
 * 	chain.add(function(){
 * 		console.log('Second!');
 * 	});
 * 	
 * 	// Run chain.
 * 	chain.run()
 */
function Chainer(){
	this.curr = undefined;
	this.chain = [];
	this.rev_chain = [];
}

// Add callback to chain.
function add(callback) {
	this.chain.push(callback);
}

// Execute from origin point in chain.
function from(origin) {
	// Convert start/end to boolean.
	origin = (origin == 'start' || origin == true);
	
	// Collect argument list after origin.
	var args = [];
	for(var i = 1; i < arguments.length; i++) {
		args.push(arguments[i]);
	}
	
	// If origin is true, we should
	// run from the start of the chain.
	if (origin) {
		// Shift everything in the reverse chain
		// back to the start of the forward chain.
		while (this.rev_chain.length) {
			this.chain.reverse();
			this.chain.push(this.rev_chain.pop());
			this.chain.reverse();
		}
		this.next.apply(this, args);
	
	// Otherwise, run from the end.
	} else {
		// Shift everything in the chain
		// to the end of the reverse chain.
		while (this.chain.length) {
			this.rev_chain.push(this.chain.shift());
		}
		this.prev.apply(this, args);
	}
}

// Execute next item in chain.
function next() {
	if (this.chain.length) {
		// Get new current function.
		var func = this.chain.shift();
		
		// Add old current function to end of rev_chain.
		if (typeof this.curr != 'undefined') {
			this.rev_chain.push(this.curr);
		}
		
		// Store new current function outside
		// of forward and reverse lists.
		this.curr = func;
		
		// Execute current function.
		func.apply(this, arguments);
	}
}

// Execute previous item in chain.
function prev() {
	if (this.rev_chain.length) {
		// Get new current function.
		var func = this.rev_chain.pop();
		
		// Add old current function to start of chain.
		if (typeof this.curr != 'undefined') {
			this.chain.reverse();
			this.chain.push(this.curr);
			this.chain.reverse();
		}
		
		// Store new current function outside
		// of forward and reverse lists.
		this.curr = func;
		
		// Execute current function.
		func.apply(this, arguments);
	}
}

Chainer.prototype.add = add;
Chainer.prototype.push = add;
Chainer.prototype.next = next;
Chainer.prototype.prev = prev;
Chainer.prototype.run = next;
Chainer.prototype.start = next;
Chainer.prototype.from = from;

module.exports = Chainer;